@@ -6,12 +6,12 @@ JMX::Jmx4Perl::Request - Encapsulates a jmx4perl request
 
 =head1 SYNOPSIS
 
-  $req = JMX::Jmx4Perl::Request->new(READ_ATTRIBUTE,$mbean,$attribute);
+  $req = JMX::Jmx4Perl::Request->new(READ,$mbean,$attribute);
 
 =head1 DESCRIPTION
 
 A L<JMX::Jmx4Perl::Request> encapsulates a request for various operational
-types. Probably the most common type is C<READ_ATTRIBUTE> which let you
+types. Probably the most common type is C<READ> which let you
 retrieve an attribute value from an MBeanServer. (In fact, at the time being,
 this is the only one supported fo now)
 
@@ -29,11 +29,11 @@ Type of request, which should be one of the constants
 
 =over
 
-=item READ_ATTRIBUTE
+=item READ
 
 Get the value of a attribute
 
-=item WRITE_ATTRIBUTE
+=item WRITE
 
 Write an attribute (not supported yet)
 
@@ -53,7 +53,7 @@ Remove a JMX notification (not supported yet)
 
 =item attribute
 
-If type is C<READ_ATTRIBUTE> or C<WRITE_ATTRIBUTE> this specifies the requested
+If type is C<READ> or C<WRITE> this specifies the requested
 attribute
 
 =item path
@@ -86,30 +86,30 @@ use Carp;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = (
-           "READ_ATTRIBUTE","WRITE_ATTRIBUTE","EXEC_OPERATION","LIST_MBEANS",
+           "READ","WRITE","EXEC_OPERATION","LIST",
            "REGISTER_NOTIFICATION","REMOVE_NOTIFICATION"
           );
 
 
 use constant {
-    READ_ATTRIBUTE => "read",
-    WRITE_ATTRIBUTE => "write",
+    READ => "read",
+    WRITE => "write",
     EXEC_OPERATION => "exec",
-    LIST_MBEANS => "list",
+    LIST => "list",
     REGISTER_NOTIFICATION => "regnotif",
     REMOVE_NOTIFICATION => "remnotif"
 };
 
 my $TYPES = 
-{ map { $_ => 1 } (READ_ATTRIBUTE, WRITE_ATTRIBUTE, EXEC_OPERATION, LIST_MBEANS, 
+{ map { $_ => 1 } (READ, WRITE, EXEC_OPERATION, LIST, 
                    REGISTER_NOTIFICATION, REMOVE_NOTIFICATION) };
 
 
 =item  $req = new JMX::Jmx4Perl::Request(....);
 
- $req = new JMX::Jmx4Perl::Request(READ_ATTRIBUTE,$mbean,$attribute,$path);
- $req = new JMX::Jmx4Perl::Request(READ_ATTRIBUTE,{ mbean => $mbean,... } );
- $req = new JMX::Jmx4Perl::Request({type => READ_ATTRIBUTE, mbean => $mbean, ... } );
+ $req = new JMX::Jmx4Perl::Request(READ,$mbean,$attribute,$path);
+ $req = new JMX::Jmx4Perl::Request(READ,{ mbean => $mbean,... } );
+ $req = new JMX::Jmx4Perl::Request({type => READ, mbean => $mbean, ... } );
 
 The constructor can be used in various way. In the simplest form, you provide
 the type as first argument and depending on the type one or more additional
@@ -124,12 +124,12 @@ named parameters are:
 
 =over
 
-=item C<READ_ATTRIBUTE>
+=item C<READ>
 
  Order    : $mbean, $attribute, $path
  Mandatory: $mbean, $attribute
 
-=item C<WRITE_ATTRIBUTE> (not supported yet)
+=item C<WRITE> (not supported yet)
 
  Order    : $mbean, $attribute, $value, $path
  Mandatory: $mbean, $attribute, $value
@@ -162,11 +162,11 @@ sub new {
         } else {
             # Unnamed arguments
             $self = {type =>  $type};
-            if ($type eq READ_ATTRIBUTE) {
+            if ($type eq READ) {
                 $self->{mbean} = shift;
                 $self->{attribute} = shift;
                 $self->{path} = shift;
-            } elsif ($type eq LIST_MBEANS) {
+            } elsif ($type eq LIST) {
                 $self->{path} = shift;
             } else {
                 croak "Type ",$type," not supported yet";
@@ -193,9 +193,9 @@ sub get {
 # Internal check for validating that all arguments are given
 sub _validate {
     my $self = shift;
-    if ($self->{type} eq READ_ATTRIBUTE || $self->{type} eq WRITE_ATTRIBUTE) {
-        croak "READ_ATTRIBUTE: No mbean name given" unless $self->{mbean};
-        croak "READ_ATTRIBUTE: No attribute name given" unless $self->{attribute};
+    if ($self->{type} eq READ || $self->{type} eq WRITE) {
+        croak "READ: No mbean name given" unless $self->{mbean};
+        croak "READ: No attribute name given" unless $self->{attribute};
     }
 }
 
